import mx.central.data.LCDataProvider;
import mx.central.controls.MDialogBox;
import mx.controls.DataGrid;
import mx.controls.gridclasses.DataGridColumn;

import mx.core.UIObject;
import mx.utils.Delegate;

/**
 * The panel in the preferences dialog that manages Alert Rules.
 * Extends UIObject to make it easier to create using a createClassObject()
 * if that is ever needed.
 */
class StockPrefsPanel extends UIObject {

    /**
	 * LCDataProvider for rules
	 */
    var oRules:LCDataProvider;

    var nCurrentRuleID:Number = -1;

    var bIsSelectionSet:Boolean;

    var oConn:IStockServer;

	var sAppName:String;
	
	var aRules:Array;
	
	var bIsInitialized:Boolean = false;

	/**
	 * References to controls on the registered MovieClip
	 */
	var fSymbol:mx.controls.TextInput;
	var fCriteriaMenu:mx.controls.ComboBox;
	var fCompareMenu:mx.controls.ComboBox;
	var fValue:mx.controls.TextInput;
	var fNoticeMenu:mx.controls.ComboBox;
	var fAddButton:mx.controls.Button;
	var fRemoveButton:mx.controls.Button;
	var fDoneButton:mx.controls.Button;
	var fAlertsGrid:mx.controls.DataGrid;		
	
	/**
	 * Reference to the MDialogBox that contains this panel.
	 * Needed when closing the preferences dialog.
	 */
	var oController:MDialogBox;
	
	var oSelectedRule:AlertRuleItem;

    /**
	 * Since this extends UIObject, init(), createChildren() and 
	 * draw() will all be called automatically, in that order.
	 */
    function init()
    {
		StockUtils.trace("init()", "StockPrefsPanel");		
	}
	
	/**
	 * Called automatically at load time, since this extends UIObject
	 */
	private function createChildren():Void
	{
		StockUtils.trace("createChildren()", "StockPrefsPanel");
		
		this.fSymbol.tabEnabled = true;
		this.fCriteriaMenu.tabEnabled = true;
		this.fCompareMenu.tabEnabled = true;
		this.fValue.tabEnabled = true;
		this.fNoticeMenu.tabEnabled = true;
		this.fAddButton.tabEnabled = true;
		this.fRemoveButton.tabEnabled = true;
		this.fDoneButton.tabEnabled = true;
		this.fAlertsGrid.tabEnabled = true;		
		
		this.fSymbol.tabIndex = 10001;
		this.fCriteriaMenu.tabIndex = 10002;
		this.fCompareMenu.tabIndex = 10003;
		this.fValue.tabIndex = 10004;
		this.fNoticeMenu.tabIndex = 10005;
		this.fAddButton.tabIndex = 10006;
		this.fRemoveButton.tabIndex = 10007;
		this.fDoneButton.tabIndex = 10008;
		this.fAlertsGrid.tabIndex = 10009;
	}
	/**
	 * Called automatically at load time, since this extends UIObject
	 */	
	private function draw():Void
	{	
		StockUtils.trace("draw()", "StockPrefsPanel");		
		if (!bIsInitialized)
		{
			this.fValue.restrict = "-1234567890.";
			
			Selection.setFocus(this.fSymbol);
			
			bIsInitialized = true;
	
			// Style values were already set by the parent app.
			// Applies styles to each TextField and UIObject in the Dialog panel.
			//StockStyles.stylizeAll(this.fDialog);
			this.fAddButton.setStyle("styleName", StockStyles.kStyleName);
			//this.fUpdateButton.setStyle("styleName", StockStyles.kStyleName);
			this.fRemoveButton.setStyle("styleName", StockStyles.kStyleName);
			this.fDoneButton.setStyle("styleName", StockStyles.kStyleName);
			
			this.fSymbol.setStyle("styleName", StockStyles.kStyleName);
			this.fValue.setStyle("styleName", StockStyles.kStyleName);
			this.fAlertsGrid.setStyle("styleName", StockStyles.kStyleName);
			this.fNoticeMenu.setStyle("styleName", StockStyles.kStyleName);
			this.fCriteriaMenu.setStyle("styleName", StockStyles.kStyleName);
			this.fCompareMenu.setStyle("styleName", StockStyles.kStyleName);

			// Event handlers
	
			// Updates the rule contents whenever a part of the rule changes.
			// Traps change events on combo boxes.
			this.fNoticeMenu.addEventListener("change", Delegate.create(this, ruleChanged));
			this.fCriteriaMenu.addEventListener("change", Delegate.create(this, ruleChanged));
			this.fCompareMenu.addEventListener("change", Delegate.create(this, ruleChanged));
			
			// Traps when the enter key is hit, and when focus is lost in a TextInput field
			this.fSymbol.addEventListener("enter", Delegate.create(this, ruleChanged));
			this.fSymbol.addEventListener("focusOut", Delegate.create(this, ruleChanged));
			this.fValue.addEventListener("enter", Delegate.create(this, ruleChanged));
			this.fValue.addEventListener("focusOut", Delegate.create(this, ruleChanged));
	
			// Next we have to make sure to update the widgets when the
			// user choses a different item in the grid
			if (this.fAlertsGrid == undefined) {
				StockUtils.trace("AlertsGrid is undefined!","StockPrefs");
			}
			
			// Traps when the selected item changes in the grid
			this.fAlertsGrid.addEventListener("change", Delegate.create(this, selectRule));
			bIsSelectionSet = false;

			// remove button
			this.fRemoveButton.addEventListener("click", Delegate.create(this, removeRule));
			
			// done button
			this.fDoneButton.addEventListener("click", Delegate.create(this, hidePreferences));
	
			// add button
			this.fAddButton.addEventListener("click", Delegate.create(this, addRule));
			
			setServer(sAppName);
			//
			// Setup grid headers
			setGridHeaders(this.fAlertsGrid);
			this.fAlertsGrid.dataProvider = oRules;
			refreshGrid();
		}
    }
	
    /**
	 * Sets up the grid's columns
	 */
    function setGridHeaders(grid:DataGrid):Void
    {
        // set grid headers
		grid.removeAllColumns();
        
		grid.setStyle("alternatingRowColors ", [ 0xDDFFDD, 0xFFFFFF ]);
        grid.columnNames = ["symbol", "criteria", "compare", "value"];

        var c:DataGridColumn;
        c = grid.getColumnAt(0);
        c.headerText = "Symbol";
        c.width = 50;

        c = grid.getColumnAt(1);
        c.headerText = "Criteria";
        c.width = 100;

        c = grid.getColumnAt(2);
        c.headerText = "";
        c.width = 70;

        c = grid.getColumnAt(3);
        c.headerText = "Value";
        c.width = 80;
    }
	
		
	public function setServer(appName:String) 
	{
		// the following instance vars were already set upon instantiation
		//sAppName = appName;
		//oConn = rulesServer;

		oRules = LCDataProvider.createClient( sAppName, "rules" );
		        
		// handle major changes in data
        var rulesListener = new Object;
		rulesListener.owner = this;
        rulesListener.modelChanged = function(eventObj:Object) {
			StockUtils.trace("rulesListener.modelChanged", "StockPrefs");
			//StockUtils.traceProps("event=",eventObj);

			// event names include updateAll, addRows, deleteRows
			if (eventObj.source != undefined) {
				this.owner.refreshGrid();
			}
        }
        oRules.addListener(rulesListener);
	}
	
	function addRule(uiEvent:Object) {
		StockUtils.trace("addRule()", "StockPrefsPanel");
		oSelectedRule = new AlertRuleItem("", "lastPrice", "over", "", AlertRuleItem.kBringToAttention);
		
		oConn.addRule(oSelectedRule);
	}
	
    function addRule_Result(newRuleID:Number):Void
    {
		StockUtils.trace("addRule_Result(" + newRuleID + ")", "StockPrefsPanel");

        selectRuleID(newRuleID);
    }
	
	function removeRule(uiEvent:Object) {
		StockUtils.trace("removeRule()", "StockPrefsPanel");
		var item = this.fAlertsGrid.selectedItem;
		if (!item.id) 
		{
			return;
		}
		if (item.id)
		{
			oConn.removeRule(item.id);
		}
	}
	
	function refreshGrid() {	
		// keep the current stock selected
		if (nCurrentRuleID > -1) {
			selectRuleID(nCurrentRuleID);
		}
	}
	
	function selectRule(uiEvent:Object) {
		StockUtils.trace("selectRule()","StockPrefsPanel");
		
		var item:AlertRuleItem = this.fAlertsGrid.selectedItem;
		//StockUtils.traceProps("selectedItem=",item);
		
		if (item != null && item.id > -1) {
			bIsSelectionSet = true;
			nCurrentRuleID = item.id;
			
			// update the ui controls
			this.fSymbol.enabled = 
			this.fValue.enabled = 
			this.fCriteriaMenu.enabled =
			this.fCompareMenu.enabled =
			this.fNoticeMenu.enabled = true;
			
			showSelectedRule();
		}
		else {
			// disable 
			bIsSelectionSet = false;
			nCurrentRuleID = -1;
			
			this.fSymbol.enabled = 
			this.fValue.enabled = 
			this.fCriteriaMenu.enabled =
			this.fCompareMenu.enabled =
			this.fNoticeMenu.enabled = false;
			this.fSymbol.text =
			this.fValue.text = "";
			// clear display here
		}
	}
	
	function showSelectedRule(uiEvent:Object) {
		StockUtils.trace("showSelectedRule()","StockPrefsPanel");
		var item:AlertRuleItem = this.fAlertsGrid.selectedItem;
		
		this.fSymbol.text = item.symbol;
		this.fValue.text = item.value;
		selectMenuData(this.fCriteriaMenu, item.criteria);
		selectMenuData(this.fCompareMenu, item.compare);
		selectMenuData(this.fNoticeMenu, item.alert);		
	}
	
	function ruleChanged(uiEvent:Object) {
		StockUtils.trace("ruleChanged(), nCurrentRuleID=" + nCurrentRuleID, "StockPrefsPanel");
		if (nCurrentRuleID > -1)
         {
			 saveRuleSettings();
		 }
	}
	
	
    function saveRuleSettings()
    {
		StockUtils.trace("saveRuleSettings()", "StockPrefs");
		var rule:AlertRuleItem = new AlertRuleItem();

		if (nCurrentRuleID > 0) {
			rule.id = nCurrentRuleID;
			rule.symbol = this.fSymbol.text.toUpperCase();
			rule.criteria = this.fCriteriaMenu.selectedItem.data;
			rule.compare = this.fCompareMenu.selectedItem.data;
			rule.value = this.fValue.text;
			rule.alert = this.fNoticeMenu.selectedItem.data;
        
	        oConn.saveRule(rule);
		}
    }
    
    function selectMenuData(menu:mx.controls.ComboBox, data)
    {
        var len:Number = menu.length;
        for (var i:Number=0; i<len; i++) {
            if (menu.getItemAt(i).data == data) {
                menu.selectedIndex = i;
                return;
            }
        }
    }
    
    // showPreferences is called by the Shell when user chooses Preferences
    function showPreferences():Void
    {
		StockUtils.trace("showPreferences()", "StockPrefs");
	
		refreshGrid();
    }
	
	function hidePreferences():Void
	{
		StockUtils.trace("hidePreferences()", "StockPrefsPanel");
		// tells the MDialogBox to close
		this.oController.doClose();
	}

    // select a rule with the given numeric ID
    function selectRuleID(ruleID:Number)
    {
		StockUtils.trace("selectRuleID(" + ruleID + ")", "StockPrefsPanel");
        var len:Number = this.fAlertsGrid.length;
        for (var i:Number=0; i<len; i++) {
            
            if (this.fAlertsGrid.getItemAt(i).id == ruleID) {
                nCurrentRuleID = ruleID;
                this.fAlertsGrid.selectedIndex = i;
                return;
            }
        }
    }

}
