/*
	FileConverter.as

	Mike Chambers
	mesh@macromedia.com

	Converts character delimited data into a an Array of Objects that can
	be converted into a mx.central.data.DataProviderClass.

	Usage:

	var fc = new FileConverter();
	fc.setTrimColumn(true);


	var lv = new LoadVars();

	lv.onData = function(data)
	{
		fc.parseData(data);
		
		dGrid.setDataProvider(fc.getDataArray());
	}

	lv.load("data.csv");

	--

	Usage (using ArrayToDataProvider class)


	var fc = new FileConverter();
	fc.setTrimColumn(true);


	var lv = new LoadVars();

	lv.onData = function(data)
	{
		fc.parseData(data);
		
		var dp = ArrayToDataProvider.parse(fc.getDataArray());

		dGrid.setDataProvider(dp);
	}

	lv.load("data.csv");
*/

class FileConverter 
{
	private var delimiter:String = ",";
	private var lineReturn:String = "\n";
	private var _dp:Array;
	private var trimColumn:Boolean = false;

	//set the column delimiter.
	public function setDelimiter(del:String):Void
	{
		this.delimiter = del;	
	}

	//set the row delimiter
	public function setLineReturn(lineRet:String):Void
	{
		this.lineReturn = lineRet;	
	}

	//specifies whether last column should be dropped. This is useful
	//if there is an extra delimiter at the end of your data.
	public function setTrimColumn(tc:Boolean):Void
	{
		this.trimColumn = tc;	
	}

	//parses the data into an Array of Objects.
	public function parseData(data:String):Void
	{

		if(data == undefined)
		{
			return;
		}

		var rows:Array = data.split(this.lineReturn);
		var rowLen:Number = rows.length;

		var columns:Array = rows[0].split(this.delimiter);

		var colMod:Number = (this.trimColumn)? 1 : 0;

		var columnsLen:Number = columns.length - colMod;

		var outArray:Array = new Array();
		var tempArray:Array;


		for(var i = 1; i < rowLen; i++)
		{
			tempArray = rows[i].split(this.delimiter);

			var obj:Object = new Object();

			for(var k = 0; k < columnsLen; k++)
			{
				obj[columns[k]] = tempArray[k];	
			}

			outArray.push(obj);
		}

		this._dp = outArray;

	}

	//returns the Array of Objects that the instance represents.
	public function getDataArray(Void):Array
	{
		return this._dp;
	}
}